<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

declare(strict_types=1);

if (!defined('_PS_VERSION_')) {
    exit;
}

if (file_exists(__DIR__ . '/vendor/autoload.php')) {
    require_once __DIR__ . '/vendor/autoload.php';
}

use Ozenie\Form\DataConfiguration\UserDataConfiguration;
use Ozenie\Form\DataConfiguration\UserUpdateDataConfiguration;
use Ozenie\Helper\Data;
use Ozenie\Service\ConnecterApi;
use Ozenie\Service\InvoiceManagement;
use PrestaShop\PrestaShop\Core\Domain\Product\ValueObject\ProductType;

class Ozenie extends Module
{
    /** @var UserDataConfiguration */
    private $userConfig;

    /** @var UserUpdateDataConfiguration */
    private $userUpdConfig;

    public function __construct()
    {
        $this->name = 'ozenie';
        $this->displayName = 'Ozénie';
        $this->tab = 'administration';
        $this->version = '1.0.0';
        $this->author = 'Ozenie';
        $this->need_instance = 0;
        $this->ps_versions_compliancy = [
            'min' => '1.6',
            'max' => _PS_VERSION_,
        ];
        $this->bootstrap = true;
		$this->module_key = '67f0061f193415ae0def33d4e2069d7c';

        parent::__construct();

        $this->displayName = $this->l('Ozenie', 'ozenie');
        $this->description = $this->l('Ozenie, API to generate factures', 'ozenie');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall?', 'ozenie');

        if (!Configuration::get('OZENIE')) {
            $this->warning = $this->l('No name provided', 'ozenie');
        }

        $this->userConfig = new UserDataConfiguration();
        $this->userUpdConfig = new UserUpdateDataConfiguration();
    }

    public function install(): bool
    {
        if (Shop::isFeatureActive()) {
            Shop::setContext(ShopCore::CONTEXT_ALL);
        }

        return
            parent::install()
            && $this->installDb() && $this->installTab()
            && $this->registerHook('actionPaymentConfirmation')
            && $this->registerHook('actionCustomerAccountUpdate')
            && $this->registerHook('actionObjectAddressUpdateAfter')
            && $this->registerHook('actionObjectProductUpdateAfter')
            && $this->registerHook('actionProductAttributeUpdate')
        ;
    }

    public function uninstall(): bool
    {
        Configuration::deleteByName($this->userConfig::LOGGED_IN);
        Configuration::deleteByName('OZ_ACCOUNT_SYNC');
        Configuration::deleteByName($this->userConfig::PWD);
        Configuration::deleteByName($this->userConfig::CREATED);
        Configuration::deleteByName('OZ_USER_PWD_REGISTER');
        Configuration::deleteByName('OZ_USER_PWD_LOGIN');
        Configuration::deleteByName($this->userConfig::ACTIVE_SUBSCRIPTION);
        Configuration::deleteByName($this->userConfig::INFO_SUBSCRIPTION);

        return (
            parent::uninstall() &&
            $this->uninstallTab() &&
            $this->uninstallDb()
        );
    }

    public function disable($force_all = false): bool
    {
        Configuration::deleteByName('OZ_USER_LOGGED_IN');

        return parent::disable($force_all);
    }

    public function installDb(): bool
    {
        return $this->executeSqlFromFile($this->getLocalPath() . 'src/Install/install.sql');
    }

    public function uninstallDb(): bool
    {
        return $this->executeSqlFromFile($this->getLocalPath() . 'src/Install/uninstall.sql');
    }

    public function executeSqlFromFile(string $path): bool
    {
        $database = Db::getInstance();
        $sqlStatements = Tools::file_get_contents($path);
        $sqlStatements = str_replace(['_DB_PREFIX_', '_MYSQL_ENGINE_'], [_DB_PREFIX_, _MYSQL_ENGINE_], $sqlStatements);

        try {
            return $database->execute($sqlStatements);
        } catch (Exception $exception) {
            return false;
        }
    }

    private function installTab(): bool
    {
        $tabId = (int) Tab::getIdFromClassName('OzenieAdminController');
        if (!$tabId) {
            $tabId = null;
        }

        try {
            $tab = new Tab($tabId);
        } catch (PrestaShopDatabaseException|PrestaShopException $e) {
            return false;
        }

        $tab->active = 1;
        $tab->class_name = 'OzenieAdminController';
        // Only since 1.7.7, you can define a route name
        $tab->route_name = 'admin_ozenie_index';
        $tab->name = array();
        foreach (Language::getLanguages() as $lang) {
            $tab->name[$lang['id_lang']] = $this->trans('Ozénie', array(), 'Modules.Ozenie.Admin', $lang['locale']);
        }
        $tab->id_parent = (int) Tab::getIdFromClassName('ShopParameters');
        $tab->module = $this->name;

        return $tab->save();
    }

    private function uninstallTab(): bool
    {
        $tabId = (int) Tab::getIdFromClassName('OzenieAdminController');
        if (!$tabId) {
            return true;
        }

        try {
            $tab = new Tab($tabId);
        } catch (PrestaShopDatabaseException|PrestaShopException $e) {
            return false;
        }

        try {
            return $tab->delete();
        } catch (PrestaShopException $e) {
            return false;
        }
    }

    /**
     * This method handles the module's configuration page.
     *
     * @return string The page's HTML content
     */
    public function getContent(): string
    {
        $output = '';
        if ($this->userConfig->isSubscriptionActive() && $this->userConfig->subscriptionInformation()) {
            $output = $this->displayInformation($this->userConfig->subscriptionInformation());
        } elseif (!$this->userConfig->isSubscriptionActive() && $this->userConfig->subscriptionInformation()) {
            $output = $this->displayError($this->userConfig->subscriptionInformation());
        }

        // this part is executed only when the form is submitted
        if (Tools::isSubmit('submitLogin')) {
            $output = $this->configUserOzenieLogin($output);
        }

        if (Tools::isSubmit('submitUpdate')) {
            $output = $this->configUserOzenieUpdate($output);
        }

        if (Tools::isSubmit('submitRegister')) {
            $output = $this->configUserOzenieRegister($output);
        }

        // display any message, then the form
        return $output . $this->displayForm();
    }

    /**
     * Builds the configuration form.
     *
     * @return string HTML code
     */
    public function displayForm(): string
    {
        $forms = [];
        $helper = new HelperForm();

        $helper->table = $this->table;
        $helper->name_controller = $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->currentIndex = AdminController::$currentIndex . '&' . http_build_query(['configure' => $this->name]);
        $helper->submit_action = 'submit' . $this->name;

        // Default language
        $helper->default_form_language = (int) Configuration::get('PS_LANG_DEFAULT');


        // Init Fields form array
        if (Configuration::get('OZ_USER_LOGGED_IN')) {

            try {
                $this->syncAccountProfil();
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog($throwable->getMessage(), 3);
            }

            $formsUpdate = $this->getUpdateUserForm();

            // Load current value into the form
            foreach ($this->userUpdConfig::LIST as $value) {
                $helper->fields_value[$value] = $this->getRequestValue($value);
            }

            $forms[] = $formsUpdate;
        } elseif (Configuration::get('OZ_USER_CREATED')) {
            $formsLogin = $this->getLoginUserForm();

            // Load current value into the form
            $helper->fields_value['OZ_USER_EMAIL_LOGIN'] = $this->getRequestValue('OZ_USER_EMAIL');
            $helper->fields_value['OZ_USER_PWD_LOGIN'] = $this->getRequestValue('OZ_USER_PWD');

            $forms[] = $formsLogin;
        } else {
            $formsLogin = $this->getLoginUserForm();
            $formsRegister = $this->getRegisterUserForm();

            // Load current value into the form
            $helper->fields_value['OZ_USER_EMAIL_LOGIN'] = $this->getRequestValue('OZ_USER_EMAIL');
            $helper->fields_value['OZ_USER_PWD_LOGIN'] = $this->getRequestValue('OZ_USER_PWD');
            $helper->fields_value['OZ_USER_EMAIL_REGISTER'] = $this->getRequestValue('OZ_USER_EMAIL');
            $helper->fields_value['OZ_USER_PWD_REGISTER'] = $this->getRequestValue('OZ_USER_PWD');
            $helper->fields_value['OZ_USER_PWD_CONF'] = $this->getRequestValue('OZ_USER_PWD_CONF');

            $forms[] = $formsLogin;
            $forms[] = $formsRegister;
        }

        // Module, token and currentIndex
        return $helper->generateForm($forms);
    }


    private function configUserOzenieRegister(string $output): string
    {
        try {
            /** @var ConnecterApi $ozenieApiService */
            $ozenieApiService = $this->get('ozenie.connecter_api');
        } catch (Exception $e) {
            // display any message, then the form
            return $output . $this->displayError($this->l($e->getMessage()));
        }

        // retrieve the value set by the user
        $config[$this->userConfig::EMAIL] = $this->getRequestValue('OZ_USER_EMAIL_REGISTER');
        $config[$this->userConfig::PWD] = $this->getRequestValue('OZ_USER_PWD_REGISTER');
        $config[$this->userConfig::PWD_CONF] = $this->getRequestValue('OZ_USER_PWD_CONF');

        try {
            // check that the value is valid
            $this->userConfig->validateConfiguration($config);
        } catch (Exception $e) {
            // invalid value, show an error
            return $output . $this->displayError($this->l($e->getMessage(), 'ozenie'));
        }

        if ($config[$this->userConfig::PWD_CONF] !== $config[$this->userConfig::PWD]) {
            return $output . $this->displayError($this->l('Password invalid', 'ozenie'));
        }

        try {
            $idEOLE = $ozenieApiService->createAccountOzenie(
                $config[$this->userConfig::EMAIL],
                $config[$this->userConfig::PWD]
            );
        } catch (Throwable $e) {
            // display any message, then the form
            return $output . $this->displayError($this->l($e->getMessage()));
        }

        if (!$idEOLE) {
            $output .= $this->displayError($this->l('Create Account in Ozenie API failed', 'ozenie'));
        } else {
            $config[$this->userConfig::ID_EOLE] = $idEOLE;
            $config[$this->userConfig::CREATED] = 1;
            // value is ok, update it and display a confirmation message
            $this->userConfig->updateConfiguration($config);

            $output .= $this->displayConfirmation($this->l(
                "Inscription réussie. De plus un mail de bienvenue contenant l’Identifiant EOLE de l’utilisateur sera envoyé à l’utilisateur",
                'ozenie'
            ));

        }

        // display any message
        return $output;
    }

    private function configUserOzenieLogin(string $output): string
    {
        try {
            /** @var ConnecterApi $ozenieApiService */
            $ozenieApiService = $this->get('ozenie.connecter_api');
        } catch (Exception $e) {
            // display any message, then the form
            return $output . $this->displayError($this->l($e->getMessage()));
        }

        // retrieve the value set by the user
        $config[$this->userConfig::EMAIL] = $this->getRequestValue('OZ_USER_EMAIL_LOGIN');
        $config[$this->userConfig::PWD] = $this->getRequestValue('OZ_USER_PWD_LOGIN');

        try {
            // check that the value is valid
            $this->userConfig->validateConfiguration($config);
        } catch (Exception $e) {
            // invalid value, show an error
            $output .= $this->displayError($this->l('Connection failed! Invalid Configuration value', 'ozenie'));
            return $output . $this->displayError($this->l($e->getMessage(), 'ozenie'));
        }

        try {
            $idEOLE = $ozenieApiService->loginAccountOzenie(
                $config[$this->userConfig::EMAIL],
                $config[$this->userConfig::PWD]
            );
        } catch (Throwable $e) {
            // display any message, then the form
            return $output . $this->displayError($this->l($e->getMessage()));
        }

        if (!$idEOLE) {
            $output .= $this->displayError($this->l('Create Account in Ozenie API failed', 'ozenie'));
        } else {
            $config[$this->userConfig::ID_EOLE] = $idEOLE;
            $config[$this->userConfig::CREATED] = 1;
            // value is ok, update it and display a confirmation message
            $this->userConfig->updateConfiguration($config);

            $output .= $this->displayConfirmation($this->l('Connection successful! Settings updated', 'ozenie'));
        }

        try {
            $infoSubscription = $ozenieApiService->infoSubscriptionOzenie();
        } catch (Throwable $e) {
            // display any message, then the form
            $output .= $this->displayError($this->l($e->getMessage()));
        }

        if (!$this->userConfig->isSubscriptionActive()) {
            return $output;
        }

        $output .= $this->displayConfirmation($infoSubscription);

        // display any message, then the form
        return $output;
    }

    private function configUserOzenieUpdate(string $output): string
    {
        try {
            /** @var ConnecterApi $ozenieApiService */
            $ozenieApiService = $this->get('ozenie.connecter_api');
        } catch (Exception $e) {
            return $output . $this->displayError($this->l($e->getMessage()));
        }

        try {
            $ozenieApiService->infoSubscriptionOzenie();
        } catch (Throwable $e) {
            $output .= $this->displayError($this->l($e->getMessage()));
        }

        if (!$this->userConfig->isSubscriptionActive()) {
            return $output;
        }


        // Retrieve the value set by the user
        $config = [];
        foreach ($this->userUpdConfig::LIST as $label) {
            $config[$label] = $this->getRequestValue($label);
        }

        try {
            // Check that the value is valid
            $this->userUpdConfig->validateConfiguration($config);
        } catch (Exception $e) {
            // Invalid value, show an error
            return $output . $this->displayError($this->l($e->getMessage(), 'ozenie'));
        }

        try {
            $updated = $ozenieApiService->updateAccountOzenie($config);
            // $idEOLE = $this->getRequestValue('OZ_USER_ID_EOLE');
        } catch (Throwable $e) {
            // display any message, then the form
            return $output . $this->displayError($this->l($e->getMessage()));
        }

        if ($updated) {
            // value is ok, update it and display a confirmation message
            $this->userUpdConfig->updateConfiguration($config);

            $output .= $this->displayConfirmation($this->l('Mise à jour du compte réussie', 'ozenie'));
        } else {
            $output .= $this->displayError($this->l('Update Account in Ozenie API failed', 'ozenie'));
        }

        // display any message, then the form
        return $output;
    }


    private function getUpdateUserForm(): array
    {
        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Ozenie API: Màj des informations du compte'),
                ],
                'input' => $this->getInputsUpdateUserForm(),
                'submit' => [
                    'title' => $this->l('Update', 'ozenie'),
                    'class' => 'btn btn-default pull-right',
                    'name'  => 'submitUpdate',
                ],
            ],
        ];
    }

    private function getSyncUserForm(): array
    {
        return [
            'form' => [
                'legend' => [
                    'title' => $this->l(
                        sprintf(
                            'Ozenie API: Synchronisation des informations du compte [ID Eole: %s]',
                            $this->getRequestValue('OZ_USER_ID_EOLE')
                        )
                    ),
                ],
                'input' => [],
                'submit' => [
                    'title' => $this->l('Synchronize', 'ozenie'),
                    'class' => 'btn btn-default pull-right',
                    'name'  => 'submitSync',
                ],
            ],
        ];
    }

    private function getLoginUserForm(): array
    {
        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Ozenie API: connection'),
                ],
                'input' => $this->getInputsLoginUserForm(),
                'submit' => [
                    'title' => $this->l('Connection', 'ozenie'),
                    'class' => 'btn btn-default pull-right',
                    'name'  => 'submitLogin',
                ],
            ],
        ];
    }

    private function getRegisterUserForm(): array
    {
        return [
            'form' => [
                'legend' => [
                    'title' => $this->l('Ozénie API: create account', 'ozenie'),
                ],
                'input' => $this->getInputsRegisterUserForm(),
                'submit' => [
                    'title' => $this->l('Register', 'ozenie'),
                    'class' => 'btn btn-default pull-right',
                    'name'  => 'submitRegister',
                ],
            ],
        ];
    }


    private function getInputsUpdateUserForm(): array
    {
        $countries = [];
        foreach (Country::getCountries(Context::getContext()->language->getId(), true) as $country) {
            $countries[] = [
                'name' => $country['name'],
                'value' => $country['name'],
            ];
        }

        return [
            [
                'col' => 3,
                'type' => 'text',
                'label' => $this->l('Raison social', 'ozenie'),
                'desc' => $this->l('Raison Sociale société'),
                'name' => 'OZ_USER_SOCIAL_REASON',
                'size' => 20,
                'required' => true,
            ], [
                'col' => 3,
                'type' => 'text',
                'label' => $this->l('Adresse', 'ozenie'),
                'desc' => $this->l('Adresse ligne 1 société'),
                'name' => 'OZ_USER_ADDRESS_1',
                'size' => 40,
                'required' => true,
            ], [
                'col' => 3,
                'type' => 'text',
                'label' => $this->l('Complément d\'adresse', 'ozenie'),
                'desc' => $this->l('Adresse ligne 2 société'),
                'name' => 'OZ_USER_ADDRESS_2',
                'size' => 20,
                'required' => false,
            ], [
                'col' => 3,
                'type' => 'text',
                'label' => $this->l('Code Postal', 'ozenie'),
                'name' => 'OZ_USER_CP',
                'size' => 20,
                'required' => true,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Ville', 'ozenie'),
                'name' => 'OZ_USER_CITY',
                'size' => 20,
                'required' => true,
            ], [
                'col' => 3,
                'type'  => 'select',
                'label' => $this->l('Pays', 'ozenie'),
                'name'  => 'OZ_USER_COUNTRY',
                'required' => true,
                'options' => [
                    'id'    => "value",
                    "name"  => "name",
                    'query' => $countries,
                ],
            ], [
                'col' => 3,
                'type'  => 'text',
                'label' => $this->l('Province', 'ozenie'),
                'desc'  => $this->l('Nom de la province pour le Canada ou USA'),
                'name'  => 'OZ_USER_PROVINCE_NAME',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Téléphone', 'ozenie'),
                'name'  => 'OZ_USER_PHONE',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Télécopie', 'ozenie'),
                'name'  => 'OZ_USER_FAX',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Mobile', 'ozenie'),
                'name'  => 'OZ_USER_MOBILE_PHONE',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Email', 'ozenie'),
                'name'  => 'OZ_USER_EMAIL',
                'size'  => 20,
                'required' => true,
                'disabled' => true,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Siret', 'ozenie'),
                'name'  => 'OZ_USER_SIRET',
                'size'  => 20,
                'required' => true,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Nom de contact', 'ozenie'),
                'name'  => 'OZ_USER_CONTACT_NAME',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('TVA Intra Communautaire', 'ozenie'),
                'name'  => 'OZ_USER_TVA_INTRA',
                'size'  => 20,
                'required' => true,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Code IBAN', 'ozenie'),
                'name'  => 'OZ_USER_IBAN',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('BIC', 'ozenie'),
                'name'  => 'OZ_USER_BIC',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('SiteWEB', 'ozenie'),
                'name'  => 'OZ_USER_WEBSITE',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('identifiant TPS', 'ozenie'),
                'desc'  => $this->l('Identifiant TPS (canada / us)'),
                'name'  => 'OZ_USER_ID_TPS',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('identifiant TVQ', 'ozenie'),
                'desc'  => $this->l('Identifiant TVQ (canada / us)'),
                'name'  => 'OZ_USER_ID_TVQ',
                'size'  => 20,
                'required' => false,
            ], [
                'col'   => 3,
                'type'  => 'text',
                'label' => $this->l('Code NAF', 'ozenie'),
                'name'  => 'OZ_USER_CODE_NAF',
                'size'  => 20,
                'required' => true,
            ], [
                'col'     => 3,
                'type'    => 'radio',
                'label'   => $this->l('Exonère TVA', 'ozenie'),
                'desc'    => $this->l('Oui si client est exonéré de TVA'),
                'name'    => 'OZ_USER_EX_TVA',
                'is_bool' => true,
                'values' => [
                    [
                        'id'    => 'no',
                        'value' => 0,
                        'label' => $this->l('No'),
                    ],
                    [
                        'id'    => 'yes',
                        'value' => 1,
                        'label' => $this->l('Yes'),
                    ],
                ],
                'required' => true,
            ],
        ];
    }

    private function getInputsLoginUserForm(): array
    {
        return [
            [
                'col' => 3,
                'type' => 'text',
                'label' => $this->l('Email', 'ozenie'),
                'name' => 'OZ_USER_EMAIL_LOGIN',
                'required' => true,
            ],
            [
                'col' => 3,
                'type' => 'password',
                'label' => $this->l('Password', 'ozenie'),
                'name' => 'OZ_USER_PWD_LOGIN',
                'desc' => "<ul>
                        <li>minimum neuf caractères</li>
                        <li>une majuscule</li>
                        <li>une minuscule</li>
                        <li>un chiffre</li>
                        <li>un caractère spécial</li>
                    </ul>",
                'required' => true,
            ],
        ];
    }

    private function getInputsRegisterUserForm(): array
    {
        return [
            [
                'col' => 3,
                'type' => 'text',
                'label' => $this->l('Email', 'ozenie'),
                'name' => 'OZ_USER_EMAIL_REGISTER',
                'required' => true,
            ],
            [
                'col' => 3,
                'type' => 'password',
                'label' => $this->l('New password', 'ozenie'),
                'name' => 'OZ_USER_PWD_REGISTER',
                'required' => true,
            ],
            [
                'col' => 3,
                'type' => 'password',
                'label' => $this->l('Confirm password', 'ozenie'),
                'name' => 'OZ_USER_PWD_CONF',
                'required' => true,
            ],
        ];
    }

    private function getLoggedInTime(): int
    {
        try {
            $datetime1 = new DateTime('@' . Configuration::get('OZ_USER_LOGGED_IN')); //start time
            $datetime2 = new DateTime(); // end time
            $interval = $datetime1->diff($datetime2);
        } catch (Exception $e) {
            return 0;
        }
        // $interval->format('%Y years %m months %d days %H hours %i minutes %s seconds');//00 years 0 months 0 days 08 hours 0 minutes 0 seconds
        return (int) $interval->format('%i');
    }

    /**
     * @return void
     * @throws Throwable
     */
    private function syncAccountProfil(): void
    {
        // Check last login time in minutes and Login
        // if more than 5 minutes
        if ($this->getLoggedInTime() > 5) {
            try {
                /** @var ConnecterApi $ozenieApiService */
                $ozenieApiService = $this->get('ozenie.connecter_api');
            } catch (Exception $e) {
                throw new Exception($e->getMessage());
            }

            try {
                $conf = $this->userConfig->getConfiguration();

                $ozenieApiService->loginAccountOzenie(
                    $conf[$this->userConfig::EMAIL], $conf[$this->userConfig::PWD]
                );
            } catch (Throwable $e) {
                throw new Exception($e->getMessage());
            }

            try {
                $ozenieApiService->infoSubscriptionOzenie();
            } catch (Throwable $e) {
                throw new Exception($e->getMessage());
            }

            try {
                $ozenieApiService->updateInformationLocal();
            } catch (Throwable $e) {
                throw new Exception($e->getMessage());
            }
        }
    }


    /**
     * Get a value from $POST / $GET if unavailable, take a config value.
     *
     * @param string $name
     *
     * @return string
     */
    private function getRequestValue(string $name): string
    {
        return (string) Tools::getValue($name, Configuration::get($name));
    }



    /**
     * Called after a payment has been validated
     */
    public function hookActionPaymentConfirmation(array $params)
    {
        if (!isset($params['id_order'])) {
            PrestaShopLogger::addLog(
                sprintf("%s: Parameter 'id_order' missing", $this->displayName),
                3
            );

            return;
        }

        $orderId = (int) $params['id_order'];

        try {
            /** @var InvoiceManagement $invoiceService */
            $invoiceService = $this->get('ozenie.invoice_management');
        } catch (Exception $e) {
            PrestaShopLogger::addLog($this->displayName . ': ' . $e->getMessage(), 3);

            return;
        }

        try {
            $invoiceService->manageInvoice($orderId);
        } catch (Throwable $throwable) {
            PrestaShopLogger::addLog(
                $this->displayName . ' manageInvoice() : ' . $throwable->getMessage(),
                3,
                null,
                Order::class, $orderId
            );

            return;
        }
    }

    /**
     * Invoked when a customer updates its account successfully
     */
    public function hookActionCustomerAccountUpdate(array $params)
    {
        if (!isset($params['customer']) || !$params['customer'] instanceof Customer) {
            PrestaShopLogger::addLog($this->displayName . ": Parameter 'customer' missing",3);

            return;
        }

        $customer = $params['customer'];

        try {
            /** @var ConnecterApi $ozenieApiService */
            $ozenieApiService = $this->get('ozenie.connecter_api');
        } catch (Exception $e) {
            PrestaShopLogger::addLog($this->displayName . ': ' . $e->getMessage(), 3);

            return;
        }

        if (
            !Configuration::get('OZ_USER_EMAIL') ||
            !Configuration::get('OZ_USER_PWD') ||
            !Configuration::get('OZ_USER_LOGGED_IN')
        ) {
            PrestaShopLogger::addLog("Variable OZ_USER_EMAIL or OZ_USER_PWD missing!", 2);

            return;
        }

        if ($ozenieApiService->getLoggedInTime() > 5) {
            try {
                $userEmail = Configuration::get('OZ_USER_EMAIL');
                $userPwd = Configuration::get('OZ_USER_PWD');
                $ozenieApiService->loginAccountOzenie($userEmail, $userPwd);
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(),3);

                return;
            }
        }

        try {
            $ozenieApiService->infoSubscriptionOzenie();
        } catch (Throwable $throwable) {
            PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(), 3);
        }

        if (!Configuration::get('OZ_ACTIVE_SUBSCRIPTION')) {
            return;
        }

        try {
            $updated = $ozenieApiService->updateClient($customer, Data::TYPE_PARTICULIER, false);
        } catch (Throwable $e) {
            PrestaShopLogger::addLog("Ozenie client update failed : " . $e->getMessage(),
                3, null,
                Customer::class, $customer->id
            );

            return;
        }

        if ($updated) {
            PrestaShopLogger::addLog(
                "Ozenie client updated", 1, null, Customer::class, $customer->id
            );
        }
    }

    public function hookActionObjectAddressUpdateAfter(array $params)
    {
        if (!isset($params['object']) && !$params['object'] instanceof Address) {
            PrestaShopLogger::addLog(
                sprintf("%s: Parameter 'Address' missing", $this->displayName),
                3
            );

            return;
        }

        $address = $params['object'];
        if (is_null($address->id_customer)) {
            return;
        }


        $customer = new Customer($address->id_customer);

        if (!Validate::isLoadedObject($customer)) {
            PrestaShopLogger::addLog(
                sprintf('%s: check object validity failed for Customer', __CLASS__),
                3, null, Customer::class, $address->id_customer
            );
        }

        PrestaShopLogger::addLog("Update Address Customer", 1, null, Address::class, $address->id);

        try {
            /** @var ConnecterApi $ozenieApiService */
            $ozenieApiService = $this->get('ozenie.connecter_api');
        } catch (Exception $e) {
            PrestaShopLogger::addLog($this->displayName . ': ' . $e->getMessage(), 3);

            return;
        }

        if (
            !Configuration::get('OZ_USER_EMAIL') ||
            !Configuration::get('OZ_USER_PWD') ||
            !Configuration::get('OZ_USER_LOGGED_IN')
        ) {
            PrestaShopLogger::addLog("Variable OZ_USER_EMAIL or OZ_USER_PWD missing!", 2);

            return;
        }

        if ($ozenieApiService->getLoggedInTime() > 5) {
            try {
                $userEmail = Configuration::get('OZ_USER_EMAIL');
                $userPwd = Configuration::get('OZ_USER_PWD');
                $ozenieApiService->loginAccountOzenie($userEmail, $userPwd);
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(),3);

                return;
            }
        }

        try {
            $ozenieApiService->infoSubscriptionOzenie();
        } catch (Throwable $throwable) {
            PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(), 3);
        }

        if (!Configuration::get('OZ_ACTIVE_SUBSCRIPTION')) {
            return;
        }

        try {
            $updated = $ozenieApiService->updateClient($customer, Data::TYPE_PARTICULIER, false);
        } catch (Throwable $e) {
            PrestaShopLogger::addLog("Ozenie client update failed : " . $e->getMessage(),
                3, null,
                Customer::class, $customer->id
            );

            return;
        }

        if ($updated) {
            PrestaShopLogger::addLog(
                "Ozenie client updated", 1, null, Customer::class, $customer->id
            );
        }
    }

    /**
     * Invoked when a product with attributes updated successfully
     */
    public function hookActionObjectProductUpdateAfter(array $params)
    {
        if (!isset($params['object']) && !$params['object'] instanceof Product) {
            PrestaShopLogger::addLog(
                sprintf("%s: Parameter 'id_product' missing", $this->displayName),
                3
            );

            return;
        }

        $product = $params['object'];

        if (
            !Configuration::get('OZ_USER_EMAIL') ||
            !Configuration::get('OZ_USER_PWD') ||
            !Configuration::get('OZ_USER_LOGGED_IN')
        ) {
            PrestaShopLogger::addLog("Variable OZ_USER_EMAIL and OZ_USER_PWD missing!", 2);

            return;
        }

        try {
            /** @var ConnecterApi $ozenieApiService */
            $ozenieApiService = $this->get('ozenie.connecter_api');
        } catch (Exception $e) {
            PrestaShopLogger::addLog($this->displayName . ': ' . $e->getMessage(), 3);

            return;
        }

        if ($ozenieApiService->getLoggedInTime() > 5) {
            try {
                $userEmail = Configuration::get('OZ_USER_EMAIL');
                $userPwd = Configuration::get('OZ_USER_PWD');
                $ozenieApiService->loginAccountOzenie($userEmail, $userPwd);
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(),3);

                return;
            }
        }

        try {
            $ozenieApiService->infoSubscriptionOzenie();
        } catch (Throwable $throwable) {
            PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(), 3);
        }

        if (!Configuration::get('OZ_ACTIVE_SUBSCRIPTION')) {
            return;
        }

        if (
            $product->product_type === ProductType::TYPE_STANDARD ||
            $product->product_type == ProductType::TYPE_VIRTUAL
        ) {

            try {
                $ozProductId = $ozenieApiService->updateArticle($product, null, false);
            } catch (Throwable $e) {
                PrestaShopLogger::addLog(
                    "Article Ozenie update failed: " . $e->getMessage(),
                    3, null, Product::class, $product->id
                );

                return;
            }

            PrestaShopLogger::addLog(
                sprintf("Article Ozenie updated (ozenieProductId:%d)", $ozProductId),
                1, null, Product::class, $product->id
            );
        }
    }

    /**
     * Invoked when a product’s attribute is updated
     */
    public function hookActionProductAttributeUpdate(array $params)
    {
        /** @var null|int $productId */
        $productId = $_POST['id_product'] ?? null;

        $productAttributeId = $params["id_product_attribute"] ?? null;

        if (!$productId || !$productAttributeId) {
            PrestaShopLogger::addLog(
                sprintf("%s: Parameter 'id_product' missing", $this->displayName),
                3
            );

            return;
        }

        if (
            !Configuration::get('OZ_USER_EMAIL') ||
            !Configuration::get('OZ_USER_PWD') ||
            !Configuration::get('OZ_USER_LOGGED_IN')
        ) {
            PrestaShopLogger::addLog("Variable OZ_USER_EMAIL and OZ_USER_PWD missing!", 2);

            return;
        }

        $product = new Product((int) $productId);

        // Validate customer object
        if (!Validate::isLoadedObject($product)) {
            PrestaShopLogger::addLog(
                sprintf('%s: check object validity failed for Product', __CLASS__),
                3, null, Product::class, $productId
            );
        }

        try {
            /** @var ConnecterApi $ozenieApiService */
            $ozenieApiService = $this->get('ozenie.connecter_api');
        } catch (Exception $e) {
            PrestaShopLogger::addLog($this->displayName . ': ' . $e->getMessage(), 3);

            return;
        }

        if ($ozenieApiService->getLoggedInTime() > 5) {
            try {
                $userEmail = Configuration::get('OZ_USER_EMAIL');
                $userPwd = Configuration::get('OZ_USER_PWD');
                $ozenieApiService->loginAccountOzenie($userEmail, $userPwd);
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(),3);

                return;
            }
        }

        try {
            $ozenieApiService->infoSubscriptionOzenie();
        } catch (Throwable $throwable) {
            PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(), 3);
        }

        if (!Configuration::get('OZ_ACTIVE_SUBSCRIPTION')) {
            return;
        }

        if ($product->product_type === ProductType::TYPE_COMBINATIONS) {

            $productAttributes = $_POST["combination_$productAttributeId"] ?? [];
            try {
                $ozProductId = $ozenieApiService->updateArticle(
                    $product, (int) $productAttributeId, false, $productAttributes
                );
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog(
                sprintf(
                    "Article Ozenie update failed (attributeId:%d): ", $productAttributeId
                ) . $throwable->getMessage(),
                3, null,
                Product::class, $product->id
                );

                return;
            }

            PrestaShopLogger::addLog(
                sprintf(
                    "Article Ozenie updated (attributeId:%d - ozenieProductId:%d)",
                    $productAttributeId,
                    $ozProductId
                ),
                1, null,
                Product::class, $product->id
            );
        }
    }


}
