<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace Ozenie\Controller\Admin;

use Context;
use Ozenie\Form\DataConfiguration\UserDataConfiguration;
use Ozenie\Form\DataConfiguration\UserUpdateDataConfiguration;
use Ozenie\Form\LoginType;
use Ozenie\Form\RegisterType;
use Ozenie\Form\UpdateType;
use Ozenie\Service\ConnecterApi;
use PrestaShopBundle\Controller\Admin\FrameworkBundleAdminController;
use PrestaShopBundle\Security\Annotation\AdminSecurity;
use PrestaShopBundle\Security\Annotation\ModuleActivated;
use Symfony\Component\HttpFoundation\Request;
use Symfony\Component\HttpFoundation\Response;

/**
 * Class OzenieAdminController.
 *
 * @ModuleActivated(moduleName="ozenie", redirectRoute="admin_module_manage")
 *
 */
class OzenieAdminController extends FrameworkBundleAdminController
{
    /**
     * @var array|mixed|null
     */
    private $idEole;

    /** @var UserDataConfiguration */
    private $userConfig;

    /** @var UserUpdateDataConfiguration */
    private $userUpdConfig;

    /** @var ConnecterApi */
    private $connecterApi;

    public function __construct(
        UserDataConfiguration $userDataConfiguration,
        UserUpdateDataConfiguration $userUpdateDataConfiguration,
        ConnecterApi $connecterApi
    ) {
        parent::__construct();
        $this->userConfig = $userDataConfiguration;
        $this->userUpdConfig = $userUpdateDataConfiguration;

        $this->idEole = $this->configuration->get($this->userConfig::ID_EOLE);
        $this->connecterApi = $connecterApi;
    }

    /**
     * @AdminSecurity("is_granted('read', request.get('_legacy_controller'))", message="Access denied.")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function indexAction(Request $request): ?Response
    {
        if (!$this->userConfig->isCreated()) {
            return $this->redirectToRoute('admin_ozenie_register');
        } elseif ($this->idEole && !$this->userConfig->isLoggedIn()) {
            return $this->redirectToRoute('admin_ozenie_login');
        } elseif ($this->userConfig->isLoggedIn()) {
            return $this->redirectToRoute('admin_ozenie_update');
        }

        return $this->render('@Modules/ozenie/views/templates/admin/index.html.twig');
    }

    /**
     * @AdminSecurity("is_granted(['read', 'update'], request.get('_legacy_controller'))", message="Access denied.")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function loginAction(Request $request): ?Response
    {
        if ($this->userConfig->isLoggedIn()) {
            $this->addFlash('error', "Already logged in Ozenie");
            return $this->redirectToRoute('admin_ozenie_update');
        }

        $form = $this->createForm(LoginType::class);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $auth = $form->getData();
            // connexion
            try {
                $this->userConfig->validateConfiguration($auth);
            } catch (\Exception $e) {
                $this->addFlash('error', $e->getMessage());
                return $this->render('@Modules/ozenie/views/templates/admin/login.html.twig', [
                    'form' => $form->createView()
                ]);
            }

            try {
                $idEole = $this->connecterApi->loginAccountOzenie(
                    $auth[$this->userConfig::EMAIL],
                    $auth[$this->userConfig::PWD]
                );
            } catch (\Throwable $throwable) {
                $this->addFlash('error', $throwable->getMessage());
                return $this->render('@Modules/ozenie/views/templates/admin/login.html.twig', [
                    'form' => $form->createView()
                ]);
            }

            $auth[$this->userConfig::CREATED] = 1;
            $auth[$this->userConfig::ID_EOLE] = $idEole;
            $this->userConfig->updateConfiguration($auth);
            $this->addFlash('success', "Connexion réussie");

            try {
                $infoSubscription = $this->connecterApi->infoSubscriptionOzenie();
                $this->addFlash('success', $infoSubscription);
            } catch (\Throwable $throwable) {
                $this->addFlash('error', $throwable->getMessage());
            }

            return $this->redirectToRoute('admin_ozenie_update');
        }

        return $this->render('@Modules/ozenie/views/templates/admin/login.html.twig', [
            'form' => $form->createView()
        ]);
    }

    /**
     * @AdminSecurity("is_granted(['read', 'update', 'create'], request.get('_legacy_controller'))", message="Access denied.")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function registerAction(Request $request): ?Response
    {
        if ($this->userConfig->isCreated()) {
            $this->addFlash('error', "Already created in Ozenie");
            return $this->redirectToRoute('admin_ozenie_login');
        }

        $form = $this->createForm(RegisterType::class);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $auth = $form->getData();
            // register
            try {
                $idEole = $this->connecterApi->createAccountOzenie(
                    $auth[$this->userConfig::EMAIL],
                    $auth[$this->userConfig::PWD]
                );
            } catch (\Throwable $throwable) {
                $this->addFlash('error', $throwable->getMessage());
                return $this->render('@Modules/ozenie/views/templates/admin/register.html.twig', [
                    'form' => $form->createView()
                ]);
            }

            $auth[$this->userConfig::CREATED] = 1;
            $auth[$this->userConfig::ID_EOLE] = $idEole;
            $this->userConfig->updateConfiguration($auth);
            $this->addFlash('success', "Connexion réussie");

            return $this->redirectToRoute('admin_ozenie_login');
        }

        return $this->render('@Modules/ozenie/views/templates/admin/register.html.twig', [
            'form' => $form->createView()
        ]);
    }

    /**
     * @AdminSecurity("is_granted(['read', 'update'], request.get('_legacy_controller'))", message="Access denied.")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function updateAction(Request $request): ?Response
    {
        if (!$this->userConfig->isCreated()) {
            $this->addFlash('error', "Aucun compte enregistré dans Ozenie");
            return $this->redirectToRoute('admin_ozenie_register');
        } elseif (!$this->userConfig->isLoggedIn()) {
            $this->addFlash('error', "Non connecté");
            return $this->redirectToRoute('admin_ozenie_register');
        }

        try {
            $infoSubscription = $this->connecterApi->infoSubscriptionOzenie();
            $this->addFlash('success', $infoSubscription);
        } catch (\Throwable $throwable) {
            $this->addFlash('error', $throwable->getMessage());
        }

        if (!$this->userConfig->isSubscriptionActive()) {
            return $this->redirectToRoute('admin_ozenie_update');
        }

        $syncLink = Context::getContext()->link->getAdminLink('OzenieAdminController', true, array(), array('ajax' => true));

        $form = $this->createForm(UpdateType::class);
        $form->handleRequest($request);
        if ($form->isSubmitted() && $form->isValid()) {
            $data = $form->getData();
            $data[$this->userUpdConfig::EMAIL] = $form->get($this->userUpdConfig::EMAIL)->getData();

            try {
                $this->userUpdConfig->validateConfiguration($data);
            } catch (\Exception $e) {
                $this->addFlash('error', $e->getMessage());
                return $this->render('@Modules/ozenie/views/templates/admin/update.html.twig', [
                    'form' => $form->createView(),
                    'syncLink' => $syncLink,
                ]);
            }

            try {
                $this->connecterApi->updateAccountOzenie($data);
            } catch (\Throwable $e) {
                $this->addFlash('error', $e->getMessage());
                return $this->render('@Modules/ozenie/views/templates/admin/update.html.twig', [
                    'form' => $form->createView(),
                    'syncLink' => $syncLink,
                ]);
            }


            $this->userUpdConfig->updateConfiguration($data);

            $this->addFlash('success', "Mise à jour du compte réussie");

            return $this->redirectToRoute('admin_ozenie_update');
        }

        return $this->render('@Modules/ozenie/views/templates/admin/update.html.twig', [
            'form' => $form->createView(),
            'syncLink' => $syncLink,
        ]);
    }

    /**
     * @AdminSecurity("is_granted(['read', 'update'], request.get('_legacy_controller'))", message="Access denied.")
     *
     * @param Request $request
     *
     * @return Response
     */
    public function syncAction(Request $request): Response
    {

        try {
            $conf = $this->userConfig->getConfiguration();

            $this->connecterApi->loginAccountOzenie(
                $conf[$this->userConfig::EMAIL], $conf[$this->userConfig::PWD]
            );
        } catch (\Throwable $throwable) {
            return new Response("Sync failed", 400);
        }

        try {
            $this->connecterApi->updateInformationLocal();
        } catch (\Throwable $throwable) {
            return new Response("Sync failed", 400);
        }

        $this->addFlash("success", 'Synchronisation du compte réussie');
        return new Response("Sync action send");
    }


}
