<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace Ozenie\Form\DataConfiguration;

use Configuration;
use Exception;
use PrestaShop\PrestaShop\Core\Configuration\DataConfigurationInterface;
use Validate;

/**
 * Handle Configuration data for user login form
 */
class UserDataConfiguration implements DataConfigurationInterface
{

    public const ID_EOLE = "OZ_USER_ID_EOLE";

    public const EMAIL = "OZ_USER_EMAIL";

    public const PWD = "OZ_USER_PWD";

    public const PWD_CONF = "OZ_USER_PWD_CONF";

    public const CREATED = "OZ_USER_CREATED";

    public const LOGGED_IN = "OZ_USER_LOGGED_IN";

    public const ACTIVE_SUBSCRIPTION = "OZ_ACTIVE_SUBSCRIPTION";

    public const INFO_SUBSCRIPTION = "OZ_INFO_SUBSCRIPTION";

    /**
     * @inheritDoc
     */
    public function getConfiguration(): array
    {
        $return = [];

        if (Configuration::get(self::ID_EOLE)) {
            $return[self::ID_EOLE] = Configuration::get(self::ID_EOLE);
        }

        if (Configuration::get(self::EMAIL)) {
            $return[self::EMAIL] = Configuration::get(self::EMAIL);
        }

        if (Configuration::get(self::PWD)) {
            $return[self::PWD] = Configuration::get(self::PWD);
        }

        if (Configuration::get(self::CREATED)) {
            $return[self::CREATED] = Configuration::get(self::CREATED);
        }

        return $return;
    }

    /**
     * @inheritDoc
     */
    public function updateConfiguration(array $configuration)
    {
        Configuration::updateValue(self::ID_EOLE, $configuration[self::ID_EOLE]);
        Configuration::updateValue(self::EMAIL, $configuration[self::EMAIL]);
        Configuration::updateValue(self::PWD, $configuration[self::PWD]);
        Configuration::updateValue(self::CREATED, $configuration[self::CREATED]);
    }

    /**
     * @inheritDoc
     * @throws Exception
     */
    public function validateConfiguration(array $configuration)
    {
        if (empty($configuration[self::EMAIL]) && !Validate::isEmail($configuration[self::EMAIL])) {
            throw new Exception("Email invalide");
        }

        $this->isStrongPassword($configuration[self::PWD]);
    }

    /**
     * Check password respect <tt>all conditions</tt> :
     *  - checks if 9 or more characters
     *  - checks if at least one digit
     *  - checks if at least one lowercase character
     *  - checks if at least one uppercase character
     *  - should not contain any white space
     *  - checks if at least one special character
     *
     * @param string $password
     *
     * @return void
     *
     * @throws Exception
     */
    private function isStrongPassword(string $password): void
    {
        // checks if 9 or more characters
        if (!Validate::isPlaintextPassword($password, 9)) {
            throw new Exception("Mot de passe invalide (minimum de 9 caractères)");
        }

        // checks if at least one digit
        if (!preg_match('/(?=.*\d)/', $password)) {
            throw new Exception("Mot de passe invalide (pas de chiffre)");
        }

        // checks if somewhere in the string is a lowercase character
        if (!preg_match('/(?=.*[a-z])/', $password)) {
            throw new Exception("Mot de passe invalide (pas de caractère minuscule)");
        }

        // checks if somewhere in the string is an uppercase character
        if (!preg_match('/(?=.*[A-Z])/', $password)) {
            throw new Exception("Mot de passe invalide (pas de caractère majuscule)");
        }

        // should not contain any white space
        if (preg_match('/(?=.*\s)/', $password)) {
            throw new Exception("Mot de passe invalide (ne doit pas contenir d'espace blanc)");
        }

        // checks if at least one special character
        if (!preg_match('/(?=.*\W)/', $password)) {
            throw new Exception("Mot de passe invalide (pas de caractère spécial)");
        }
    }

    /**
     * Get/Update value
     *
     * @param int|null $isLogin
     * @return false|string
     */
    public function isLoggedIn(int $isLogin = null)
    {
        if (!is_null($isLogin)) {
            return Configuration::updateValue(self::LOGGED_IN, $isLogin);
        }

        return Configuration::get(self::LOGGED_IN);
    }

    /**
     * Get/Update value
     *
     * @param bool|null $isCreated
     *
     * @return false|string
     */
    public function isCreated(bool $isCreated = null)
    {
        if (!is_null($isCreated)) {
            return Configuration::updateValue(self::CREATED, $isCreated);
        }

        return Configuration::get(self::CREATED);
    }

    /**
     * Check if subscription active (in configuration)
     *
     * @return false|string
     */
    public function isSubscriptionActive()
    {
        return Configuration::get(self::ACTIVE_SUBSCRIPTION);
    }

    /**
     * Get information subscription (in configuration)
     *
     * @return false|string
     */
    public function subscriptionInformation()
    {
        return Configuration::get(self::INFO_SUBSCRIPTION);
    }
}
