<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

namespace Ozenie\Form\DataConfiguration;

use Configuration;
use Exception;
use PrestaShop\PrestaShop\Core\Configuration\DataConfigurationInterface;
use Validate;

class UserUpdateDataConfiguration implements DataConfigurationInterface
{
    public const SOCIAL_REASON = 'OZ_USER_SOCIAL_REASON';
    public const ADDRESS_1 = 'OZ_USER_ADDRESS_1';
    public const POSTAL_CODE = 'OZ_USER_CP';
    public const CITY = 'OZ_USER_CITY';
    public const COUNTRY = 'OZ_USER_COUNTRY';
    public const EMAIL = 'OZ_USER_EMAIL';
    public const SIRET = 'OZ_USER_SIRET';
    public const TVA_INTRA = 'OZ_USER_TVA_INTRA';
    public const CODE_NAF = 'OZ_USER_CODE_NAF';
    public const EX_TVA = 'OZ_USER_EX_TVA';

    public const ADDRESS_2 = 'OZ_USER_ADDRESS_2';
    public const PROVINCE_NAME = 'OZ_USER_PROVINCE_NAME';
    public const PHONE = 'OZ_USER_PHONE';
    public const FAX = 'OZ_USER_FAX';
    public const MOBILE_PHONE = 'OZ_USER_MOBILE_PHONE';
    public const CONTACT_NAME = 'OZ_USER_CONTACT_NAME';
    public const IBAN = 'OZ_USER_IBAN';
    public const BIC = 'OZ_USER_BIC';
    public const WEBSITE = 'OZ_USER_WEBSITE';
    public const ID_TPS = 'OZ_USER_ID_TPS';
    public const ID_TVQ = 'OZ_USER_ID_TVQ';

    public const LIST = [
        self::SOCIAL_REASON,
        self::ADDRESS_1,
        self::POSTAL_CODE,
        self::CITY,
        self::COUNTRY,
        self::EMAIL,
        self::SIRET,
        self::TVA_INTRA,
        self::CODE_NAF,
        self::EX_TVA,
        self::ADDRESS_2,
        self::PROVINCE_NAME,
        self::PHONE,
        self::FAX,
        self::MOBILE_PHONE,
        self::CONTACT_NAME,
        self::IBAN,
        self::BIC,
        self::WEBSITE,
        self::ID_TPS,
        self::ID_TVQ,
    ];

    public const LIST_EXTERNAL_KEY = [
        self::SOCIAL_REASON => 'RaisonSociale',
        self::ADDRESS_1 => 'Adresse1',
        self::ADDRESS_2 => 'Adresse2',
        self::POSTAL_CODE => 'CodePostal',
        self::CITY => 'Ville',
        self::COUNTRY => 'NomPays',
        self::EMAIL => 'eMail',
        self::SIRET => 'NumeroSiretSiren',
        self::TVA_INTRA => 'TVAIntraCommunautaire',
        self::CODE_NAF => 'CodeNAF',
        self::EX_TVA => 'ExonereTVA',
        self::PROVINCE_NAME => 'NomProvince',
        self::PHONE => 'Telephone',
        self::FAX => 'Telecopie',
        self::MOBILE_PHONE => 'Mobile',
        self::CONTACT_NAME => 'NomContact',
        self::IBAN => 'CodeIBAN',
        self::BIC => 'BIC',
        self::WEBSITE => 'SiteWEB',
        self::ID_TPS => 'idEntifiantTPS',
        self::ID_TVQ => 'identifiantTVQ',
    ];


    /**
     * @inheritDoc
     */
    public function getConfiguration(): array
    {
        $return = [];

        $return[self::EMAIL] = Configuration::get(self::EMAIL) ?: "";
        $return[self::ADDRESS_1] = Configuration::get(self::ADDRESS_1) ?: "";
        $return[self::ADDRESS_2] = Configuration::get(self::ADDRESS_2) ?: "";
        $return[self::POSTAL_CODE] = Configuration::get(self::POSTAL_CODE) ?: "";
        $return[self::CITY] = Configuration::get(self::CITY) ?: "";
        $return[self::COUNTRY] = Configuration::get(self::COUNTRY) ?: "";
        $return[self::PROVINCE_NAME] = Configuration::get(self::PROVINCE_NAME) ?: "";
        $return[self::PHONE] = Configuration::get(self::PHONE) ?: "";
        $return[self::MOBILE_PHONE] = Configuration::get(self::MOBILE_PHONE) ?: "";
        $return[self::SOCIAL_REASON] = Configuration::get(self::SOCIAL_REASON) ?: "";
        $return[self::BIC] = Configuration::get(self::BIC) ?: "";
        $return[self::CODE_NAF] = Configuration::get(self::CODE_NAF) ?: "";
        $return[self::CONTACT_NAME] = Configuration::get(self::CONTACT_NAME) ?: "";
        $return[self::EX_TVA] = Configuration::get(self::EX_TVA) ?: "";
        $return[self::FAX] = Configuration::get(self::FAX) ?: "";
        $return[self::IBAN] = Configuration::get(self::IBAN) ?: "";
        $return[self::ID_TPS] = Configuration::get(self::ID_TPS) ?: "";
        $return[self::ID_TVQ] = Configuration::get(self::ID_TVQ) ?: "";
        $return[self::SIRET] = Configuration::get(self::SIRET) ?: "";
        $return[self::TVA_INTRA] = Configuration::get(self::TVA_INTRA) ?: "";
        $return[self::WEBSITE] = Configuration::get(self::WEBSITE) ?: "";

        return $return;
    }

    /**
     * @inheritDoc
     */
    public function updateConfiguration(array $configuration): array
    {
        Configuration::updateValue(self::EMAIL, $configuration[self::EMAIL]);
        Configuration::updateValue(self::ADDRESS_1, $configuration[self::ADDRESS_1]);
        Configuration::updateValue(self::ADDRESS_2, $configuration[self::ADDRESS_2]);
        Configuration::updateValue(self::POSTAL_CODE, $configuration[self::POSTAL_CODE]);
        Configuration::updateValue(self::CITY, $configuration[self::CITY]);
        Configuration::updateValue(self::COUNTRY, $configuration[self::COUNTRY]);
        Configuration::updateValue(self::PROVINCE_NAME, $configuration[self::PROVINCE_NAME]);
        Configuration::updateValue(self::PHONE, $configuration[self::PHONE]);
        Configuration::updateValue(self::MOBILE_PHONE, $configuration[self::MOBILE_PHONE]);
        Configuration::updateValue(self::SOCIAL_REASON, $configuration[self::SOCIAL_REASON]);
        Configuration::updateValue(self::BIC, $configuration[self::BIC]);
        Configuration::updateValue(self::CODE_NAF, $configuration[self::CODE_NAF]);
        Configuration::updateValue(self::CONTACT_NAME, $configuration[self::CONTACT_NAME]);
        Configuration::updateValue(self::EX_TVA, (bool) $configuration[self::EX_TVA]);
        Configuration::updateValue(self::FAX, $configuration[self::FAX]);
        Configuration::updateValue(self::IBAN, $configuration[self::IBAN]);
        Configuration::updateValue(self::ID_TPS, $configuration[self::ID_TPS]);
        Configuration::updateValue(self::ID_TVQ, $configuration[self::ID_TVQ]);
        Configuration::updateValue(self::SIRET, $configuration[self::SIRET]);
        Configuration::updateValue(self::TVA_INTRA, $configuration[self::TVA_INTRA]);
        Configuration::updateValue(self::WEBSITE, $configuration[self::WEBSITE]);

        return $configuration;
    }

    /**
     * @inheritDoc
     * @throws Exception
     */
    public function validateConfiguration(array $configuration)
    {
        if (
            empty($configuration[self::SOCIAL_REASON]) ||
            !Validate::isGenericName($configuration[self::SOCIAL_REASON])
        ) {
            throw new Exception("La raison social est invalide", 3);
        }

        if (
            empty($configuration[self::ADDRESS_1]) ||
            !Validate::isAddress($configuration[self::ADDRESS_1])
        ) {
            throw new Exception("L'adresse est invalide", 3);
        }

        if (
            empty($configuration[self::POSTAL_CODE]) ||
            !Validate::isPostCode($configuration[self::POSTAL_CODE])
        ) {
            throw new Exception("Le code postal est invalide", 3);
        }

        if (
            empty($configuration[self::CITY]) ||
            !Validate::isGenericName($configuration[self::CITY])
        ) {
            throw new Exception("La ville est invalide", 3);
        }

        if (
            empty($configuration[self::COUNTRY]) ||
            !Validate::isGenericName($configuration[self::COUNTRY])
        ) {
            throw new Exception("Le pays est invalide", 3);
        }

        if (
            empty($configuration[self::EMAIL]) ||
            !Validate::isEmail($configuration[self::EMAIL])
        ) {
            throw new Exception("Email invalide", 3);
        }

        if (
            empty($configuration[self::SIRET]) ||
//            !Validate::isSiret($configuration[self::SIRET])
            !Validate::isGenericName($configuration[self::SIRET])
        ) {
            throw new Exception("Siret invalide", 3);
        }

        if (
            empty($configuration[self::CODE_NAF]) ||
            !Validate::isApe($configuration[self::CODE_NAF])
        ) {
            throw new Exception("Code NAF invalide", 3);
        }

        if (!Validate::isBool($configuration[self::EX_TVA])) {
            throw new Exception("Exonère TVA invalide", 3);
        }

        if (!Validate::isFloat($configuration[self::TVA_INTRA])) {
            throw new Exception("TVA Intra Communautaire invalide", 3);
        }
    }
}
