<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

declare(strict_types=1);

namespace Ozenie\Service;

use Category;
use Configuration;
use Context;
use Currency;
use Customer;
use DateTime;
use Exception;
use Gender;
use Module;
use Order;
use Ozenie\Form\DataConfiguration\UserDataConfiguration;
use Ozenie\Form\DataConfiguration\UserUpdateDataConfiguration;
use Ozenie\Helper\Data;
use Ozenie\Repository\OzCustomerRepository;
use Ozenie\Repository\OzProductRepository;
use PaymentModule;
use PrestaShop\PrestaShop\Core\Domain\Product\ValueObject\ProductType;
use PrestaShopLogger;
use Product;
use Throwable;
use Validate;

class ConnecterApi
{
    public const TYPE_PAYMENT_CB = "Transfert bancaire";

    public const TYPE_PAYMENT_PAYPAL = "PayPal";

    public const TYPE_PAYMENT_CHECK = "Chèque";

    /** @var false|string */
    private $idEole;

    /** @var OzProductRepository */
    private $ozProductRepository;

    /** @var OzCustomerRepository */
    private $ozCustomerRepository;

    /** @var string */
    private $url;

    /** @var UserDataConfiguration */
    private $userConfig;

    /** @var UserUpdateDataConfiguration */
    private $userUpdConfig;

    public function __construct(
        string $url,
        OzProductRepository $ozProductRepository,
        OzCustomerRepository $ozCustomerRepository
    ) {
        $this->url = $url;
        $this->ozProductRepository = $ozProductRepository;
        $this->ozCustomerRepository = $ozCustomerRepository;
        $this->userConfig = new UserDataConfiguration();
        $this->userUpdConfig = new UserUpdateDataConfiguration();
        $this->idEole = Configuration::get($this->userConfig::ID_EOLE);

    }

    /**
     * @param string $method
     * @param string $uri
     * @param array $options
     *
     * @return array
     *
     * @throws Throwable
     */
    private function request(string $method, string $uri, array $options = []): array
    {
        $defaultOptions = [
            'headers' => [
                'Accept' => 'application/json',
            ],
        ];

        // merges the default options with the passed options
        $options = array_merge_recursive($defaultOptions, $options);

        $route = '/rest/' . trim($uri, '/');
        $url = rtrim($this->url) . '/' . ltrim($route, '/');

		if ($method === 'POST') {
			$response = $this->postRequest($url, $options);
		} elseif ($method === 'GET') {
			$response = $this->getRequest($url);
		} else {
	        throw new Exception("Method not found!");
		}

		return $response;
    }

	/**
	 * GET request
	 *
	 * @throws Exception
	 */
	private function getRequest(string $url): array
	{
		$ch = curl_init();
		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt($ch, CURLOPT_TIMEOUT, 5);
		$data = curl_exec($ch);

		$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

		curl_close($ch);

		if ($httpCode>=200 && $httpCode<300) {
			return json_decode($data, true) ?? [];
		}

		throw new Exception("[$httpCode] Erreur lors de l'envoie de la requête (GET) sur l'API Ozenie");
	}

	/**
	 * POST request
	 *
	 * @throws Exception
	 */
	private function postRequest(string $url, array $fields): array
	{
		$ch = curl_init();
		$data = $fields['json'];
		$dataString = json_encode($data);

		curl_setopt($ch, CURLOPT_URL, $url);
		curl_setopt($ch, CURLOPT_HTTPHEADER, array(
			'Content-Type:application/json',
			'Content-Length: ' . strlen($dataString)
		));
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, 1);
		curl_setopt($ch, CURLOPT_CONNECTTIMEOUT, 5);
		curl_setopt($ch, CURLOPT_TIMEOUT, 5);

		if(!empty($fields)){
			curl_setopt($ch, CURLOPT_POST, true);
			curl_setopt($ch, CURLOPT_POSTFIELDS, $dataString);
		}

		$data = curl_exec($ch);

		if (curl_errno($ch)) {
			throw new Exception(curl_error($ch));
		}

		$httpCode = curl_getinfo($ch, CURLINFO_HTTP_CODE);

		curl_close($ch);

		if ($httpCode>=200 && $httpCode<300) {
			return json_decode($data, true) ?? [];
		}

		throw new Exception("[$httpCode] Erreur lors de l'envoie de la requête (POST) sur l'API Ozenie");
	}

    /**
     * Create Account Ozenie.
     *
     * @throws Throwable
     *
     * @return false|string
     */
    public function createAccountOzenie(string $email, string $password)
    {
        $response = $this->request('POST', 'compte/creerCompte', [
            'json' => [
                'email'      => $email,
                'motDePasse' => $password,
            ],
        ]);

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            throw new Exception($response['DetailReponse']);
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        return $response['DetailReponse'];
    }

    /**
     * @throws Throwable
     *
     * @return false|string
     */
    public function loginAccountOzenie(string $email, string $password)
    {
        $response = $this->request('POST', 'compte/connexion', [
            'json' => [
                'email'      => $email,
                'motDePasse' => $password,
            ],
        ]);

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            Configuration::updateValue('OZ_USER_LOGGED_IN', false);

            throw new Exception($response['DetailReponse']);
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        Configuration::updateValue('OZ_USER_LOGGED_IN', time());

        return $response['DetailReponse'];
    }

    /**
     * @throws Throwable
     */
    public function infoSubscriptionOzenie(): string
    {
        if (!$this->idEole) {
            PrestaShopLogger::addLog('Missing config "clé api ozenie" in module Ozenie', 3);

            throw new Exception('Missing config "clé api ozenie" in module Ozenie');
        }

        $response = $this->request('POST', 'module/infoModuleUtilisateur', [
            'json' => [
                'identifiantEOLE'   => $this->idEole,
                'identifiantModule' => 28, // identifiantModule: Prestashop
            ],
        ]);

        if (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        $detail = $response["DetailReponse"];

        if ($response['Reponse'] === false) {

            if ($response['CodeReponse'] === '0030') {
                Configuration::updateValue('OZ_ACTIVE_SUBSCRIPTION', 0);
                Configuration::updateValue('OZ_INFO_SUBSCRIPTION', $detail);
                throw new Exception($detail);
            } elseif ($response['CodeReponse'] === '0027') {

                Configuration::updateValue('OZ_ACTIVE_SUBSCRIPTION', 0);
                Configuration::updateValue('OZ_INFO_SUBSCRIPTION', sprintf(
                    'Abonnement non résilié et abonnement expirée le <b>%s</b>',
                    (new DateTime($detail))->format('d/m/y')
                ));

                throw new Exception(sprintf(
                    'Abonnement non résilié et abonnement expirée le <b>%s</b>',
                    (new DateTime($detail))->format('d/m/y')
                ));
            } elseif ($response['CodeReponse'] === '0028') {

                Configuration::updateValue('OZ_ACTIVE_SUBSCRIPTION', 1);
                Configuration::updateValue('OZ_INFO_SUBSCRIPTION', sprintf(
                    'Abonnement résilié et abonnement non expirée (le <b>%s</b>)',
                    (new DateTime($detail))->format('d/m/y')
                ));

                throw new Exception(
                    sprintf(
                        'Abonnement résilié et abonnement non expirée (le <b>%s</b>)',
                        (new DateTime($detail))->format('d/m/y')
                    )
                );
            } elseif ($response['CodeReponse'] === '0029') {

                Configuration::updateValue('OZ_ACTIVE_SUBSCRIPTION', 0);
                Configuration::updateValue('OZ_INFO_SUBSCRIPTION', sprintf(
                    'Abonnement résilié et abonnement expirée le <b>%s</b>',
                    (new DateTime($detail))->format('d/m/y')
                ));

                throw new Exception(
                    sprintf(
                        'Abonnement résilié et Abonnement expirée le <b>%s</b>',
                        (new DateTime($detail))->format('d/m/y')
                    )
                );
            }
        }

        Configuration::updateValue('OZ_ACTIVE_SUBSCRIPTION', 1);
        Configuration::updateValue('OZ_INFO_SUBSCRIPTION', sprintf(
            'Vous êtes abonnés. Date d\'expiration de l\'abonnement <b>%s</b>.',
            (new DateTime($detail))->format('d/m/y')
        ));

        return sprintf(
            "Vous êtes abonnés. Date d'expiration de l'abonnement <b>%s</b>.",
            (new DateTime($detail))->format('d/m/Y')
        );
    }

    /**
     * Update Account Ozenie informations. (local)
     *
     * @throws Throwable
     */
    public function updateInformationLocal(): bool
    {
        if (!$this->idEole) {
            PrestaShopLogger::addLog('Missing config "clé api ozenie" in module Ozenie', 3);

            throw new Exception('Missing config "clé api ozenie" in module Ozenie');
        }

        $response = $this->request('GET', sprintf(
            'compte/infosUtilisateur/%s', $this->idEole
        ));

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            throw new Exception($response['DetailReponse']);
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        $informations = $response['DetailReponse'];

        $config=[];
        foreach ($this->userUpdConfig::LIST_EXTERNAL_KEY as $key => $externalKey) {
            if (!isset($informations[$externalKey])) {
                continue;
            }
            $config[$key] = $informations[$externalKey];
        }

        $this->userUpdConfig->updateConfiguration($config);

        Configuration::updateValue('OZ_USER_LOGGED_IN', time());
        Configuration::updateValue('OZ_ACCOUNT_SYNC', true);

        return true;
    }

    /**
     * Update Account Ozenie.
     *
     * @param array $informations
     *
     * @return bool
     * @throws Throwable
     */
    public function updateAccountOzenie(array $informations): bool
    {
        if (!$this->idEole) {
            PrestaShopLogger::addLog('Missing config "clé api ozenie" in module Ozenie', 3);

            throw new Exception('Missing config "clé api ozenie" in module Ozenie');
        }

        $conf = $this->userUpdConfig;

        $body = [
            'IdentifiantEOLE'       => $this->idEole,
            'RaisonSociale'         => $informations[$conf::SOCIAL_REASON],
            'Adresse1'              => $informations[$conf::ADDRESS_1],
            'Adresse2'              => $informations[$conf::ADDRESS_2] ?? "", // optionnel
            'CodePostal'            => $informations[$conf::POSTAL_CODE],
            'Ville'                 => $informations[$conf::CITY],
            'NomPays'               => $informations[$conf::COUNTRY],
            'NomProvince'           => $informations[$conf::PROVINCE_NAME] ?? "", // optionnel
            'Telephone'             => $informations[$conf::PHONE] ?? "", // optionnel
            'Telecopie'             => $informations[$conf::FAX] ?? "", // optionnel
            'Mobile'                => $informations[$conf::MOBILE_PHONE] ?? "", // optionnel
            'eMail'                 => $informations[$conf::EMAIL],
            'NumeroSiretSiren'      => $informations[$conf::SIRET],
            'NomContact'            => $informations[$conf::CONTACT_NAME] ?? "", // optionnel
            'TVAIntraCommunautaire' => $informations[$conf::TVA_INTRA],
            'CodeIBAN'              => $informations[$conf::IBAN] ?? "", // optionnel
            'BIC'                   => $informations[$conf::BIC] ?? "", // optionnel
            'SiteWEB'               => $informations[$conf::WEBSITE] ?? "", // optionnel
            'idEntifiantTPS'        => $informations[$conf::ID_TPS] ?? "", // optionnel
            'identifiantTVQ'        => $informations[$conf::ID_TVQ] ?? "", // optionnel
            'CodeNAF'               => $informations[$conf::CODE_NAF],
            'ExonereTVA'            => (bool) $informations[$conf::EX_TVA],
        ];

        $response = $this->request('POST', 'compte/maj', ['json' => $body]);

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            throw new Exception($response['DetailReponse']);
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        return true;
    }

    /**
     * Ajoute un client dans la base de données du compte Ozénie
     *
     * @param Customer $customer
     * @param int $type Customer type: Ozenie\Helper\Data::CUSTOMER_TYPE[particulier, association, professional]
     * @param int $addressId Optional. Address id
     *
     * @return int Return customerExternalId
     *
     * @throws Throwable
     */
    public function addClient(Customer $customer, int $type, int $addressId): int
    {
        if (!$this->idEole) {
            PrestaShopLogger::addLog('Missing config "clé api ozenie" in module Ozenie', 3);

            throw new Exception('Missing config "clé api ozenie" in module Ozenie');
        }

        $idLang = Context::getContext()->language->getId();
        $gender = new Gender($customer->id_gender);
        $genderName = is_array($gender->name) ? reset($gender->name) : $gender->name;
        $birthday = ($customer->birthday) ? (new DateTime($customer->birthday))->format('Ymd') : "";

        $addressesInfos = $customer->getSimpleAddress($addressId, $idLang);

        $label = $addressesInfos['alias'] ?? $customer->lastname . '  ' . $customer->firstname;

        $body = [
            "IdentifiantEOLE" => $this->idEole, // required
            "TypeClient" => $type, // 1 = professionnel | 2 = association | 3 = particulier  // required
            "Designation" => Data::addSpaceInString($label),  // required
            "Adresse1" => Data::addSpaceInString($addressesInfos['address1']),
            "Adresse2" => Data::addSpaceInString($addressesInfos['address2']),
            "CodePostal" => $addressesInfos['postcode'],  // required
            "Ville" => $addressesInfos['city'],  // required
            "Telephone" => $addressesInfos['phone'],
            "Mobile" => $addressesInfos['phone_mobile'],
            "CodeClient" => '', // Code comptable du client
            "Email" => $customer->email,
            "Commentaire" => "commentaire",
            "DateEntree" => (new DateTime())->format('Ymd'),  // required
            "Pays" => $addressesInfos['country'],  // required
            "eMailFactureClient" => "",
            "NomProvince" => $addressesInfos['state'] ?? "",
            "NomClient" => ($type === Data::CUSTOMER_TYPE['particulier']) ? $customer->lastname . " " : "",
            "PrenomClient" => ($type === Data::CUSTOMER_TYPE['particulier']) ? $customer->firstname . " " : "",
            "NomContactSociete" => "",
            "TVAIntracom" => ($type === Data::CUSTOMER_TYPE['professional']) ? $addressesInfos['vat_number']  : "",
            "NumeroSIRET" => ($type === Data::CUSTOMER_TYPE['professional']) ? $customer->siret : "",
            "CodeNAF" => $customer->ape ?: "",
            "eMailPro" => "",
            "Civilite" => $genderName,
            "SansTVA" => empty($addressesInfos['vat_number']),
            "DateNaissanceComplete"=> $birthday,
        ];

        $response = $this->request('POST', 'client/ajoutClient', ['json' => $body]);

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            throw new Exception($response['DetailReponse']);
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        Configuration::updateValue('OZ_USER_LOGGED_IN', time());

        return $response['DetailReponse'];
    }

    /**
     * Update Client.
     *
     * - The customer id if the modification was successful OR the imported customer id (numeric) if this
     * customer did not exist and the user chose the import option if the customer did not exist not in its base
     * - False if there was an error
     *
     * @throws Throwable
     */
    public function updateClient(Customer $customer, int $type, bool $allowAddition, ?int $addressId=null): bool
    {
        if (!$this->idEole) {
            PrestaShopLogger::addLog('Missing config "clé api ozenie" in module Ozenie', 3);

            throw new Exception('Missing config "clé api ozenie" in module Ozenie');
        }

        $customer = new Customer($customer->id);


        $ozCustomer = $this->ozCustomerRepository->findOneBy(['customerId' => $customer->id]);

        if (is_null($ozCustomer)) {
            throw new Exception(
                sprintf("Ozenie customer not found in local database (id:%d)", $customer->id)
            );
        }

        // Validate customer object
        if (!Validate::isLoadedObject($customer)) {
            throw new Exception(sprintf(
                '%s: check object validity failed for Customer (id:%d)',
                __CLASS__,
                $customer->id
            ));
        }

        $idLang = Context::getContext()->language->getId();
        $gender = new Gender($customer->id_gender);
        $genderName = is_array($gender->name) ? reset($gender->name) : $gender->name;
        $birthday = ($customer->birthday) ? (new DateTime($customer->birthday))->format('Ymd') : "";

        if (is_null($addressId)) {
            $addresses = $customer->getAddresses($idLang);
            $addressesInfos = reset($addresses);
        } else {
            $addressesInfos = $customer->getSimpleAddress($addressId, $idLang);
        }

        $label = $addressesInfos['alias'] ?? $customer->lastname . '  ' . $customer->firstname;

        $body = [
            "IdentifiantEOLE"   => $this->idEole, // required
            "IdentifiantClient" => $ozCustomer->getExternalId(),
            "AutorisAjout" => $allowAddition, // required
            "TypeClient"   => $type, // 1 = professionnel | 2 = association | 3 = particulier  // required
            "Designation"  => $label, // required
            "Adresse1"   => $addressesInfos['address1'],
            "Adresse2"   => $addressesInfos['address2'],
            "CodePostal" => $addressesInfos['postcode'], // required
            "Ville"      => $addressesInfos['city'], // required
            "Telephone"  => $addressesInfos['phone'],
            "Mobile"     => $addressesInfos['phone_mobile'],
            "CodeClient" => '', // Code comptable du client
            "Email"      => $customer->email,
            "Commentaire"   => "commentaire",
            "DateEntree"    => (new DateTime())->format('Ymd'),  // required
            "Pays"          => $addressesInfos['country'],  // required
            "eMailPro"  => "",
            "eMailFactureClient" => "",
            "NomContactSociete"  => "",
            "NomProvince"   => $addressesInfos['state'] ?? "",
            "NomClient"     => ($type === Data::CUSTOMER_TYPE['particulier']) ? $customer->lastname . " " : "",
            "PrenomClient"  => ($type === Data::CUSTOMER_TYPE['particulier']) ? $customer->firstname : "",
            "TVAIntracom" => ($type === Data::CUSTOMER_TYPE['professional']) ? $addressesInfos['vat_number'] : "",
            "NumeroSIRET" => ($type === Data::CUSTOMER_TYPE['professional']) ? $customer->siret : "",
            "CodeNAF"   => $customer->ape ?: "",
            "Civilite"  => $genderName,
            "SansTVA"   => empty($addressesInfos['vat_number']),
            "DateNaissanceComplete"=> $birthday,
        ];

        $response = $this->request('POST', 'client/modifierClient', ['json' => $body]);

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            $stringJson = json_encode($body);
            throw new Exception($response['DetailReponse'] . " : <code>" . var_export($stringJson, true) . "</code>");
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        Configuration::updateValue('OZ_USER_LOGGED_IN', time());

        return true; // customerId | false
    }

    /**
     * import a user's item record into the database.
     * Data allocated to the user on Ozenie.
     *
     * @param array $product Product infos
     *
     * @throws Throwable
     */
    public function addArticle(array $product): int
    {
        if (!$this->idEole) {
            PrestaShopLogger::addLog('Missing config "clé api ozenie" in module Ozenie', 3);

            throw new Exception('Missing config "clé api ozenie" in module Ozenie');
        }

        $idLang = Context::getContext()->language->getId();
        $name = is_array($product['name']) ? reset($product['name']) : $product['name'];
        $productLabel = ($product['attributes']) ? sprintf("$name (%s)", $product['attributes']) : $name;
        $productCategoryName = is_array($product['category']) ? reset($product['category']) : $product['category'];

        $article = [
            'IdentifiantEOLE'   => $this->idEole,
            'CodeArticle'       => $product['reference'],
            'LibelleArticle'    => Data::addSpaceInString($productLabel),
            'PrixAchatHT'       => (float) $product['wholesale_price'],
            'PrixVenteHT'       => (float) $product['price_with_reduction_without_tax'],
            'TauxTVA'           => (float) $product['rate'],
            'TauxTVA2'          => '0',
            'ArticleRevente'    => false,
            'ArticlePRestation' => false,
            'UniteVente'        => $product['unity'],
            'Fournisseur'       => (string) $product['supplier_reference'],
            'Categorie'         => $productCategoryName,
        ];

        $response = $this->request('POST', 'article/ajoutArticle', ['json' => $article]);

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            throw new Exception($response['DetailReponse']);
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        return $response['DetailReponse'];
    }

    /**
     * Modify an article of the user in his database on Ozenie.
     * But also to import if he tries to modify an article file that does not exist on Ozenie.
     *
     * @param Product $product
     * @param int|null $combinaisonId
     * @param bool $allowAddition
     * @param array $productAttributes
     *
     * @return int
     *
     * @throws Throwable
     */
    public function updateArticle(Product $product, ?int $combinaisonId, bool $allowAddition, array $productAttributes = []): int
    {
        if (!$this->idEole) {
            throw new Exception('Missing config "clé api ozenie" in module Ozenie');
        }

        $ozProduct = $this->ozProductRepository->findOneBy(['productId' => $product->id, 'productAttributeId' => $combinaisonId]);
        if (is_null($ozProduct)) {
            throw new Exception('OzProduct not found');
        }

        $idLang = Context::getContext()->language->getId();
        $productName = is_array($product->name) ? reset($product->name) : $product->name;
        $productCat = new Category($product->id_category_default, $idLang);
        $productCategoryName = is_array($productCat->name) ? reset($productCat->name) : $productCat->name;
        $salesPriceWithoutTax = $product->getPrice(
            false, $combinaisonId,
            2,
            null,
            false,
            false
        );

        $combinaisonsAttrList = "";
        if ($combinaisonId) {
            $combinaisonsAttrList = $this->getProductCombinaisonsAttrList($combinaisonId, $product, $idLang);

            if ($productAttributes) {
                $salesPriceWithoutTax = $product->getPrice(
                    false, null, 2, null, false, false
                );
                $salesPriceWithoutTax = $salesPriceWithoutTax + (float) $productAttributes['attribute_price'];

            }
        }

        $article = [
            'IdentifiantEOLE'    => $this->idEole,
            "IdentifiantArticle" => $ozProduct->getExternalId(),
            "AutorisAjout"      => $allowAddition,
            'CodeArticle'       => $product->reference,
            'LibelleArticle'    => Data::addSpaceInString("$productName $combinaisonsAttrList"),
            'PrixAchatHT'       => (float) $product->wholesale_price,
            'PrixVenteHT'       => $salesPriceWithoutTax,
            'TauxTVA'           => $product->getTaxesRate(),
            'TauxTVA2'          => '0',
            'ArticleRevente'    => false,
            'ArticlePRestation' => false,
            'UniteVente'        => $product->unity,
            'Fournisseur'       => $product->supplier_name,
            'Categorie'         => $productCategoryName,
        ];

        $response = $this->request('POST', 'article/modifierArticle', ['json' => $article]);

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            throw new Exception($response['DetailReponse']);
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        return $ozProduct->getExternalId();
    }

    /**
     * Fonction qui permet d’importer une facture de l’utilisateur dans sa base de données sur
     * Ozénie et également de la générer si souhaité(envoie mail au client et à l’expéditeur).
     *
     * @param Order $order
     * @param int $ozCustomerId
     * @param array $products
     * @param bool $sendEmail
     * @param string $senderEmail
     *
     * @return int
     *
     * @throws Throwable
     */
    public function addInvoice(Order $order, int $ozCustomerId, array $products, bool $sendEmail, string $senderEmail): int
    {
        if (!$this->idEole) {
            PrestaShopLogger::addLog('Missing config "clé api ozenie" in module Ozenie', 3);

            throw new Exception('Missing config "clé api ozenie" in module Ozenie');
        }

        // Calc VAT amount
        $vatAmount = ($order->getTaxesAverageUsed() / 100) * $order->total_paid_tax_excl;

        $invoiceDetails = $this->getOrderItems($order, $products);

        $customerPaymentInfos = $this->getCustomerPaymentInfos($order);

        $invoice = [
            "IdentifiantEOLE"   => $this->idEole,
            "IdentifiantClient" => $ozCustomerId,
            "MailExpediteur"    => $senderEmail,
            "EnvoiMail"         => $sendEmail,
            "CorpsMail"         => Data::addSpaceInString("Veuillez-trouvez ci-joint votre facture"),
            'Entetefacture' => [
                'NumeroCommande'   => $order->reference,
                'DatePiece'        => (new DateTime())->format('Ymd'), // AAAAMMJJ,
                'MontantRemise'    => (float) $order->total_discounts,
                'MontantFraisPort' => (float) $order->total_shipping,
                'MontantHT'        => (float) $order->total_paid_tax_excl,
                'MontantTTC'       => (float) $order->total_paid_tax_incl,
                'MontantTVA'       => (float) $vatAmount, // Tax and shipping cost
                'MontantReglement' => (float) $order->total_paid,
                'RemisedePied'     => 0,
                'RemisedePiedMt'   => 0,
                "ObjetFacture"     => "Facture",
                'DetailFacture'    => $invoiceDetails,
                'ReglementClient'  => $customerPaymentInfos,
            ],
        ];

        $response = $this->request('POST', 'facture/ajoutFacture', ['json' => $invoice]);

        if (isset($response['Reponse']) && $response['Reponse'] === false) {
            throw new Exception($response['DetailReponse']);
        } elseif (!isset($response['Reponse'])) {
            throw new Exception("Erreur dans la réponse de l'API Ozenie");
        }

        Configuration::updateValue('OZ_USER_LOGGED_IN', time());

        // return identifiantFacture
        return $response['DetailReponse'];
    }

    /**
     * Get order items.
     * @param Order $order
     * @param array $products List product with Ozenie product_external_id
     * @return array
     */
    private function getOrderItems(Order $order, array $products): array
    {
        $oi = [];
        $cartProducts = $order->getCartProducts();

        foreach ($cartProducts as $cp) {
            $ozProductId=null;

            foreach ($products as $product) {
                if ($cp['product_type'] === ProductType::TYPE_COMBINATIONS) {
                    if (
                        $product['id_product'] === (int) $cp['id_product'] &&
                        $product['id_product_attribute'] === (int) $cp['product_attribute_id']
                    ) {
                        $ozProductId = $product['product_external_id'];
                    }
                } else {
                    if ($product['id_product'] === (int) $cp['id_product']) {
                        $ozProductId = $product['product_external_id'];
                    }
                }
            }

            if (!$ozProductId) {
                continue;
            }

            $oi[] = [
                'QuantiteCommande' => (int) $cp['cart_quantity'],
                'PrixUnitaire'     => (float) $cp['unit_price_tax_excl'],
                'PrixUnitaireTTC'  => round($cp['unit_price_tax_incl'], 2),
                'MontantHT'        => (float) $cp['total_price_tax_excl'],
                'MontantTTC'       => (float) $cp['total_price_tax_incl'],
                'TauxRemise'       => (float) $cp['reduction_percent'],
                'MontantRemise'    => (float) $cp['reduction_amount'],
                'QuantiteLivre'    => 0,
                'QuantiteFacture'  => (int) $cp['product_quantity_in_stock'],
                'TypePiece'        => $cp['product_type'],
                'Prestation'       => false,
                'Vente'            => true,
                'TxPourboire'      => 0,
                'MontantPourboire' => 0,
                'Gratuit'          => ((float) $cp['product_price'] == 0),
                'InfoArticle'      => [
                    'IDArticle' => (int) $ozProductId,
                ],
            ];
        }

        return $oi;
    }

    /**
     * Get customer payment information.
     *
     * @param Order $order
     *
     * @return array
     *
     * @throws Exception
     */
    private function getCustomerPaymentInfos(Order $order): array
    {
        $currency = $this->loadCurrencyById((int) $order->id_currency);
        $paymentMethod = $this->getPaymentMethod($order);

        return [
            'LibelleReglement' => $paymentMethod,
            'DateReglement'    => (new DateTime($order->date_add))->format('Ymd'),
            'MontantReglement' => (float) $order->total_paid,
            'Commentaire'      => $order->note,
            'CodeDevise'       => $currency->iso_code,
        ];
    }

    /**
     * @throws Exception
     */
    private function loadCurrencyById(int $id_currency): Currency
    {
        $currency = new Currency($id_currency);

        // Validate currency object
        if (Validate::isLoadedObject($currency)) {
            return $currency;
        }

        throw new Exception("Currency loaded invalid in ConnecterApi::loadCurrencyById()");
    }

    /**
     * @param Order $order
     * @return string
     */
    private function getPaymentMethod(Order $order): string
    {
        switch ($order->payment) {
            case self::TYPE_PAYMENT_CB:
                $paymentMethod = "CB";
                break;
            case self::TYPE_PAYMENT_PAYPAL:
                $paymentMethod = "PAY";
                break;
            case self::TYPE_PAYMENT_CHECK:
                $paymentMethod = "";
                break;
            default:
                $paymentMethod = $order->payment;
        }

        return $paymentMethod;
    }

    /**
     * @return array
     */
    private function getListPaymentMethod(): array
    {
        $paymentMethods = array();

        foreach (PaymentModule::getInstalledPaymentModules() as $payment) {
            $module = Module::getInstanceByName($payment['name']);
            if (Validate::isLoadedObject($module) && $module->active) {
                $paymentMethods[] = $module->displayName;
            }
        }

        return $paymentMethods;
    }

    /**
     * @param int|null $combinaisonId
     * @param Product $product
     * @param int $idLang
     *
     * @return string
     */
    private function getProductCombinaisonsAttrList(?int $combinaisonId, Product $product, int $idLang): string
    {
        $attributeCombinations = ($combinaisonId) ? $product->getAttributeCombinationsById($combinaisonId, $idLang) : [];

        if (empty($attributeCombinations)) {
            return "";
        }

        $combinaisons = "(";
        $i=0;
        foreach ($attributeCombinations as $ac) {
            if ($i > 0) {
                $combinaisons .= ", ";
            }

            $combinaisons .= $ac['group_name'] . ": " . $ac['attribute_name'];
            $i++;
        }

        $combinaisons .= ")";

        return $combinaisons;
    }



    /**
     * Get Logged In Time
     */
    public function getLoggedInTime(): int
    {
        try {
            $datetime1 = new DateTime('@' . Configuration::get('OZ_USER_LOGGED_IN')); //start time
            $datetime2 = new DateTime(); // end time
            $interval = $datetime1->diff($datetime2);
        } catch (Exception $e) {
            return 0;
        }
        // $interval->format('%Y years %m months %d days %H hours %i minutes %s seconds');//00 years 0 months 0 days 08 hours 0 minutes 0 seconds
        return (int) $interval->format('%i');
    }
}
