<?php
/**
 * Copyright since 2007 PrestaShop SA and Contributors
 * PrestaShop is an International Registered Trademark & Property of PrestaShop SA
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Open Software License (OSL 3.0)
 * that is bundled with this package in the file LICENSE.md.
 * It is also available through the world-wide-web at this URL:
 * https://opensource.org/licenses/OSL-3.0
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to https://devdocs.prestashop.com/ for more information.
 *
 * @author    PrestaShop SA and Contributors <contact@prestashop.com>
 * @copyright Since 2007 PrestaShop SA and Contributors
 * @license   https://opensource.org/licenses/OSL-3.0 Open Software License (OSL 3.0)
 */

declare(strict_types=1);

namespace Ozenie\Service;

use Cart;
use Configuration;
use Context;
use Customer;
use Order;
use Ozenie\Entity\OzCustomer;
use Ozenie\Helper\Data;
use Ozenie\Repository\OzCustomerRepository;
use Ozenie\Repository\OzInvoiceRepository;
use Ozenie\Repository\OzProductRepository;
use PrestaShopLogger;
use Product;
use Throwable;
use Validate;

class InvoiceManagement
{
    /** @var ConnecterApi */
    private $connecterApi;

    /** @var OzProductRepository */
    private $ozProductRepository;

    /** @var OzCustomerRepository */
    private $ozCustomerRepository;

    /** @var OzInvoiceRepository */
    private $ozInvoiceRepository;

    public function __construct(
        ConnecterApi $connecterApi,
        OzProductRepository $ozProductRepository,
        OzCustomerRepository $ozCustomerRepository,
        OzInvoiceRepository $ozInvoiceRepository
    ) {
        $this->connecterApi = $connecterApi;
        $this->ozProductRepository = $ozProductRepository;
        $this->ozCustomerRepository = $ozCustomerRepository;
        $this->ozInvoiceRepository = $ozInvoiceRepository;
    }

    /**
     * @throws Throwable
     */
    public function manageInvoice(int $orderId): void
    {

        if (
            !Configuration::get('OZ_USER_ID_EOLE') ||
            !Configuration::get('OZ_USER_CREATED') ||
            !Configuration::get('OZ_USER_LOGGED_IN')
        ) {
            PrestaShopLogger::addLog(sprintf('%s: not created/logIn in API', __CLASS__), 2);

            return;
        }

        PrestaShopLogger::addLog(
            sprintf('%s: Generate invoice', __CLASS__),
            1, null,
            Order::class, $orderId
        );

        if ($this->connecterApi->getLoggedInTime() > 5) {
            try {
                $userEmail = Configuration::get('OZ_USER_EMAIL');
                $userPwd = Configuration::get('OZ_USER_PWD');
                $this->connecterApi->loginAccountOzenie($userEmail, $userPwd);
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(), 3);

                return;
            }

        }

        try {
            $this->connecterApi->infoSubscriptionOzenie();
        } catch (Throwable $throwable) {
            PrestaShopLogger::addLog(__CLASS__ . ' : ' . $throwable->getMessage(), 3);
        }

        if (!Configuration::get('OZ_ACTIVE_SUBSCRIPTION')) {
            return;
        }

        $order = $this->loadOrderById((int) $orderId);
        if (is_null($order)) {
            return;
        }

        $customer = $this->loadCustomerById((int) $order->id_customer);
        if (is_null($customer)) {
            return;
        }

        $cart = $this->loadCartById((int) $order->id_cart);
        if (is_null($cart)) {
            return;
        }

        $error = false;
        $i=0;
        $cartProducts = [];
        foreach ($cart->getProducts(false, false, null, true) as $cartProduct) {

            $cartProducts[$i]['id_product'] = $productId = (int) $cartProduct['id_product'];
            $productAttributeId = null;
            if (isset($cartProduct['id_product_attribute']) && $cartProduct['id_product_attribute']) {
                $cartProducts[$i]['id_product_attribute'] = $productAttributeId = (int) $cartProduct['id_product_attribute'];
            }

            $ozProduct = $this->ozProductRepository->findOneBy(['productId' => $productId, 'productAttributeId' => $productAttributeId]);

            if ($ozProduct) {
                $cartProducts[$i]['product_external_id'] = $ozProduct->getExternalId();
                $i++;
                continue;
            }

            try {
                $externalId = $this->connecterApi->addArticle($cartProduct);
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog(
                    sprintf('%s: addArticle() : ', __CLASS__) . $throwable->getMessage(),
                    3, null, Product::class, $productId
                );

                $i++;
                $error = true;
                continue;
            }

            if ($externalId) {
                $cartProducts[$i]['product_external_id'] = $externalId;
                $this->ozProductRepository
                    ->insertProduct($productId, $productAttributeId, $externalId)
                ;
            }

            $i++;
        }

        if ($error) {
            return;
        }

        $ozCustomer = $this->ozCustomerRepository->findOneBy(['customerId'=>$customer->id]);

        if (is_null($ozCustomer)) {
            try {
                $customerExternalId = $this->connecterApi->addClient(
                    $customer, Data::TYPE_PARTICULIER,
                    (int) $cart->id_address_invoice
                );
            } catch (Throwable $throwable) {
                PrestaShopLogger::addLog(
                    sprintf('%s: addClient() : ', __CLASS__) . $throwable->getMessage(),
                    3, null,
                    Customer::class, $customer->id
                );

                return;
            }

            if ($customerExternalId) {
                $this->ozCustomerRepository->insertCustomer($customer->id, $customerExternalId);
            }
        } else {
            $customerExternalId = $ozCustomer->getExternalId();
            if (count($customer->getAddresses(Context::getContext()->language->getId())) > 1) {
                $this->connecterApi->updateClient(
                    $customer, Data::TYPE_PARTICULIER, false, (int)$cart->id_address_invoice
                );
            }
        }

        $ozInvoice = $this->ozInvoiceRepository->findOneBy(['orderId'=>$order->id]);

        if (is_null($ozInvoice)) {
            try {
                $invoiceExternalId = $this->connecterApi->addInvoice($order, (int)$customerExternalId, $cartProducts, true, 'florent@seyes.re');
            } catch (Throwable $e) {
                PrestaShopLogger::addLog(
                    "Ozenie invoice creation failed: " . $e->getMessage(),
                    3, null,
                    Order::class, $order->id
                );

                return;
            }

            $this->ozInvoiceRepository->insertInvoice($order->id, $invoiceExternalId);

            PrestaShopLogger::addLog(
                "Ozenie invoice created",
                1, null,
                Order::class, $order->id
            );
        }
    }

    /**
     * @throws Throwable
     */
    private function loadOrderById(int $orderId): ?Order
    {
        // Load order object
        $order = new Order($orderId);

        // Validate Order object
        if (Validate::isLoadedObject($order)) {
            return $order;
        }

        PrestaShopLogger::addLog(sprintf('%s: Order (id:%d) missing', __CLASS__, $orderId),3);

        return null;
    }

    private function loadCustomerById(int $customerId): ?Customer
    {
        $customer = new Customer($customerId);

        // Validate customer object
        if (Validate::isLoadedObject($customer)) {
            return $customer;
        }

        PrestaShopLogger::addLog(sprintf('%s: Customer (id:%d) missing', __CLASS__, $customerId),3);

        return null;
    }

    private function loadCartById(int $cartId): ?Cart
    {
        $cart = new Cart($cartId);

        // Validate customer object
        if (Validate::isLoadedObject($cart)) {
            return $cart;
        }

        PrestaShopLogger::addLog(sprintf('%s: Cart (id:%d) missing', __CLASS__, $cartId),3);

        return null;
    }
}
